/**
 * Copyright (c) 2008-2010 Capgemini, 2008-2012 EBMWebsourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.component.framework.constants;

import java.net.URI;

import org.ow2.petals.component.framework.api.Message.MEPConstants;

/**
 * SOAP Version Constants for the Message Exchange Patterns.
 * @author Christophe DENEUX - Capgemini Sud
 */
public enum BindingSoapVersion {
    SOAP11_NS("http://schemas.xmlsoap.org/wsdl/soap/"), SOAP12_NS(
            "http://schemas.xmlsoap.org/wsdl/soap12/");

    /**
     * 
     * @param pattern
     * @return
     */
    public static BindingSoapVersion valueOf(final URI pattern) {
        BindingSoapVersion result = null;
        if (pattern != null) {
            for (final BindingSoapVersion soap : BindingSoapVersion.values()) {
                if (soap.nameSpace.equals(pattern.toString())) {
                    result = soap;
                }
            }
        }
        return result;
    }

    private final String nameSpace;

    private final URI soapURI;

    /**
     * Creates a new instance of {@link MEPConstants}
     * 
     * @param nameSpace
     */
    private BindingSoapVersion(final String nameSpace) {
        this.nameSpace = nameSpace;
        this.soapURI = URI.create(nameSpace);
    }

    /**
     * 
     * @return
     */
    public URI value() {
        return this.soapURI;
    }
}
