/**
 * Copyright (c) 2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.component.framework.monitoring.probe;

import java.util.List;

import javax.management.MBeanNotificationInfo;

import org.ow2.petals.component.framework.process.MessageExchangeAcceptor;
import org.ow2.petals.probes.api.probes.GaugeProbe;

/**
 * Interface of the probe about the message exchange acceptor pool
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public interface MessageExchangeAcceptorPoolProbe extends GaugeProbe<Integer, Integer> {

    /**
     * The message exchange acceptor pool probed.
     * 
     * @param acceptorPool
     *            The message exchange acceptor pool probed.
     */
    public void setAcceptorPool(final List<MessageExchangeAcceptor> acceptorPool);

    /**
     * Set the expected size of the message exchange acceptor pool probed.
     * 
     * @param acceptorPoolSize
     *            The expected size of the message exchange acceptor pool
     *            probed.
     */
    public void setAcceptorPoolSize(final long acceptorPoolSize);

    /**
     * Create the information associated to the JMX notifications representing
     * the defects thrown by this probe
     * 
     * @return Information associated to the JMX notifications representing the
     *         defects thrown by this probe
     */
    public MBeanNotificationInfo[] getNotificationInfo();

}
