/**
 * Copyright (c) 2012 EBMWebsourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.component.framework.process;

import java.util.logging.Logger;

import javax.jbi.messaging.MessageExchange;

import org.ow2.petals.component.framework.AbstractComponent;
import org.ow2.petals.component.framework.jbidescriptor.generated.Settablepositivestrictint;
import org.ow2.petals.component.framework.jbidescriptor.generated.Settablepositivestrictlong;

/**
 * This class helps a Component to listen to {@link MessageExchange}s on the
 * DeliveryChannel (accept() method).
 * <p>
 * It starts or stops a pool of thread that listen to new incoming JBI from the
 * bus. {@link MessageExchange}s.
 * </p>
 * 
 * @author Christophe DENEUX - EBM WebSourcing
 * @since 5.3
 */
public abstract class AbstractMessageManager {

    /**
     * Retry policy: default value of the max number of tries.
     */
    protected static final int DEFAULT_ACCEPTOR_RETRY_MAX_TRY = 40;

    /**
     * Retry policy: default value of the base duration between two tries, in
     * milliseconds.
     */
    protected static final int DEFAULT_ACCEPTOR_RETRY_BASE_DURATION = 250;

    protected final Logger logger;

    protected final AbstractComponent component;

    /**
     * Creates a new instance of {@link AbstractMessageManager}
     * 
     * @param component
     * @param logger
     */
    public AbstractMessageManager(final AbstractComponent component, final Logger logger) {
        super();
        this.component = component;
        this.logger = logger;
    }

    public final int getRetryMaxTry() {
        final Settablepositivestrictint retryMaxTry = this.component.getComponentConfiguration()
                .getAcceptorRetryNumber();
        int retryMaxTryValue;
        if (retryMaxTry == null) {
            retryMaxTryValue = DEFAULT_ACCEPTOR_RETRY_MAX_TRY;
        } else {
            retryMaxTryValue = retryMaxTry.getValue();
            if (retryMaxTryValue <= 0) {
                retryMaxTryValue = DEFAULT_ACCEPTOR_RETRY_MAX_TRY;
                this.logger
                        .warning("Invalid value for attribute 'acceptor-retry-number': The value must be upper or equals to 1. Default value used ("
                                + DEFAULT_ACCEPTOR_RETRY_MAX_TRY + ").");
            }
        }
        return retryMaxTryValue;
    }

    public final long getRetryBaseDuration() {
        final Settablepositivestrictlong retryBaseDuration = this.component
                .getComponentConfiguration().getAcceptorRetryWait();
        long retryBaseDurationValue;
        if (retryBaseDuration == null) {
            retryBaseDurationValue = DEFAULT_ACCEPTOR_RETRY_BASE_DURATION;
        } else {
            retryBaseDurationValue = retryBaseDuration.getValue();
            if (retryBaseDurationValue <= 0) {
                retryBaseDurationValue = DEFAULT_ACCEPTOR_RETRY_BASE_DURATION;
                this.logger
                        .warning("Invalid value for attribute 'acceptor-retry-number': The value must be upper or equals to 1. Default value used ("
                                + DEFAULT_ACCEPTOR_RETRY_BASE_DURATION + ").");
            }
        }

        return retryBaseDurationValue;
    }
}
