/**
 * Copyright (c) 2006-2012 EBMWebsourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.component.framework.process;

import java.util.concurrent.atomic.AtomicInteger;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.ow2.petals.component.framework.AbstractComponent;

/**
 * The Acceptor factory used to create new Acceptors
 * 
 * @author Christophe HAMERLING - EBM WebSourcing
 * @since 3.0
 * 
 */
public class JBIAcceptorThreadFactory {

    private final AtomicInteger threadNumber = new AtomicInteger(1);

    private final ThreadGroup group;

    private final Logger logger;

    private final AbstractComponent component;

    private final JBIProcessorManager jbiProcessorManager;

    public JBIAcceptorThreadFactory(final AbstractComponent component,
            final JBIProcessorManager jbiProcessorManager, final Logger logger) {
        this.component = component;
        this.jbiProcessorManager = jbiProcessorManager;
        this.logger = logger;
        this.group = new ThreadGroup(Thread.currentThread().getThreadGroup(), this.component
                .getContext().getComponentName() + "-JBIListener");
    }

    /**
     * @param nbMaxTry
     *            Number of tries that an acceptor will submit a message
     *            exchange processor for processing before to declare that it
     *            cannot be processed.
     * @param baseWaitDuration
     *            Base duration, in milliseconds, that an acceptor will wait
     *            between two processing submission tries.
     * @return A new message exchange acceptor as a thread.
     */
    public MessageExchangeAcceptor newThread(final int nbMaxTry, final long baseWaitDuration) {
        final String name = this.component.getContext().getComponentName()
                + " -JBI Acceptor Thread #" + Integer.toString(this.threadNumber.getAndIncrement());

        this.logger.log(Level.FINE, "Creating a new thread for JBIAcceptor Factory : " + name);

        final MessageExchangeAcceptor meaProcessor = new MessageExchangeAcceptor(this.component,
                this.jbiProcessorManager,
                this.logger, this.group, name, nbMaxTry, baseWaitDuration);
        meaProcessor.setDaemon(true);
        return meaProcessor;
    }

    @Override
    protected void finalize() throws Throwable {
        // Deregistration of the thread group
        this.group.destroy();
    }
}
