/**
 * Copyright (c) 2008-2012 EBMWebsourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.component.framework.process;

import java.util.concurrent.ThreadFactory;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.ow2.petals.component.framework.AbstractComponent;

/**
 * JBI message processor thread factory
 * 
 * @author Christophe DENEUX (EBMWebsourcing)
 */
class JBIProcessorThreadFactory implements ThreadFactory {

    private final AtomicInteger threadNumber = new AtomicInteger(1);

    private final String componentName;

    private final AbstractComponent component;

    private final Logger logger;

    /**
     * Create a new instance of {@link JBIListenerThreadFactory}
     * @param jbiProcessorManager TODO
     * 
     * @param componentName
     * @param logger
     */
    public JBIProcessorThreadFactory(final AbstractComponent component, final Logger logger) {
        this.component = component;
        this.logger = logger;
        this.componentName = this.component.getComponentConfiguration()
                .getIdentification().getName();
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.util.concurrent.ThreadFactory#newThread(java.lang.Runnable)
     */
    public Thread newThread(final Runnable r) {
        final String name = this.componentName + " - MessageExchange Processor thread #"
                + Integer.toString(this.threadNumber.getAndIncrement());
        this.logger.log(Level.FINE, "Creating new JBI Processor Thread : "
                + name);
        return new Thread(Thread.currentThread().getThreadGroup(), r, name);
    }
}