/**
 * Copyright (c) 2008-2012 EBMWebsourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.component.framework.process;

import java.util.logging.Logger;

import org.apache.commons.pool.BasePoolableObjectFactory;
import org.ow2.petals.component.framework.AbstractComponent;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.listener.AbstractJBIListener;

import com.ebmwebsourcing.easycommons.lang.StringHelper;

/**
 * JBI Processor Factory
 * 
 * @author Christophe DENEUX (EBMWebsourcing)
 * 
 */
class MessageExchangeProcessorObjectFactory extends BasePoolableObjectFactory {
    
    private final AbstractComponent component;

    private final Logger logger;

    public MessageExchangeProcessorObjectFactory(final AbstractComponent component, final Logger logger) {
        this.component = component;
        this.logger = logger;
    }

    public Object makeObject() throws Exception {
        final AbstractJBIListener jbiListener = this.createAndInitJBIListener();

        final MessageExchangeProcessor jbiProcessor = new MessageExchangeProcessor(
                this.component, jbiListener,
                this.logger);
        return jbiProcessor;
    }

    /**
     * Instantiate and initialize a JBI listener.s This JBI Listener is the
     * one which is defined by the component developer.
     * 
     * @return
     * @throws PEtALSCDKException
     */
    private AbstractJBIListener createAndInitJBIListener() throws PEtALSCDKException {
        AbstractJBIListener jbiListener = null;
        final String jbiListenerClassNameValue = this.component
                .getComponentConfiguration().getJbiListenerClassName();
        final String jbiListenerClassName = jbiListenerClassNameValue.replaceAll("\\s", "");

        if (!StringHelper.isNullOrEmpty(jbiListenerClassName)) {
            try {
                final ClassLoader componentClassLoader = this.getClass().getClassLoader();
                final Class<?> clazz = componentClassLoader.loadClass(jbiListenerClassName);
                jbiListener = (AbstractJBIListener) clazz.newInstance();
            } catch (final InstantiationException e) {
                throw new PEtALSCDKException("Can't instanciate JBIListener : "
                        + jbiListenerClassName, e);
            } catch (final IllegalAccessException e) {
                throw new PEtALSCDKException("Can't instanciate JBIListener : "
                        + jbiListenerClassName, e);
            } catch (final ClassNotFoundException e) {
                throw new PEtALSCDKException("Can't find JBIListener class : "
                        + jbiListenerClassName, e);
            }
            jbiListener.init(this.component);
            // component implementor specific init
            jbiListener.init();
        } else {
            throw new PEtALSCDKException("JBIListener class name can not be empty");
        }
        return jbiListener;
    }

}