/**
 * Copyright (c) 2008-2012 EBMWebsourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.component.framework.process.async;

import java.util.Map;
import java.util.Map.Entry;
import java.util.TimerTask;
import java.util.logging.Logger;

import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.message.ExchangeImpl;
import org.ow2.petals.component.framework.process.JBIProcessorManager;

/**
 * The cleaner task. Remove all the expired exchanges.
 */
public class AsyncContextCleaner extends TimerTask {

    private final Map<String, AsyncContextExchange> asynchronousContexts;

    private final JBIProcessorManager jbiProcessorManager;

    /**
     * Number of tries to submit a message exchange processor for processing
     * before to declare that it cannot be processed.
     */
    private final int nbMaxTry;

    /**
     * Base duration, in milliseconds, to wait between two processing submission
     * tries.
     **/
    private final long baseWaitDuration;

    private final Logger logger;

    /**
     * 
     * @param originalAsynchronousContexts
     * @param asynchronousContexts
     * @param jbiProcessorManager
     * @param logger
     */
    public AsyncContextCleaner(final Map<String, AsyncContextExchange> asynchronousContexts,
            final JBIProcessorManager jbiProcessorManager, final int nbMaxTry,
            final long baseWaitDuration, final Logger logger) {
        super();
        this.asynchronousContexts = asynchronousContexts;
        this.jbiProcessorManager = jbiProcessorManager;
        this.nbMaxTry = nbMaxTry;
        this.baseWaitDuration = baseWaitDuration;
        this.logger = logger;
    }

    @Override
    public void run() {

        try {
            for (final Entry<String, AsyncContextExchange> asyncContextExchangeEntry : this.asynchronousContexts
                    .entrySet()) {
                final AsyncContextExchange asyncContextExchange = asyncContextExchangeEntry.getValue();
                final AsyncContext asyncContext = asyncContextExchange.asyncContext;
                final long expirationTime = asyncContext.getExpirationTime();
                if (expirationTime != 0 && System.currentTimeMillis() > expirationTime) {
                    // A message sent asynchronously has expired
                    final Exchange exchange = asyncContextExchange.exchange;
                    exchange.setProperty(
                            AsyncMessageManager.ASYNC_EXPIRED_MESSAGE_PROPERTY, Boolean.TRUE);

                    this.jbiProcessorManager.process((ExchangeImpl) exchange, this.nbMaxTry,
                            this.baseWaitDuration);
                }
            }
        } catch (final InterruptedException e1) {
            this.logger.fine("The processing of expired asynchronous messages is interrupted.");
        }
    }
}
