/**
 * Copyright (c) 2005-2012 EBMWebsourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.component.framework.su;

import java.util.List;

import javax.jbi.management.DeploymentException;

import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.bc.AbstractBindingComponent;
import org.ow2.petals.component.framework.jbidescriptor.generated.Consumes;
import org.ow2.petals.component.framework.process.ExternalListenerManager;

/**
 * Like SimpleServiceUnitManager but start and stop listeners for external addresses.
 * @author Olivier Fabre - EBM WebSourcing
 */
public class BindingComponentServiceUnitManager extends AbstractServiceUnitManager {

    private final ExternalListenerManager externalListenerManager;

    public BindingComponentServiceUnitManager(AbstractBindingComponent bindingComponent) {
        super(bindingComponent);
        this.externalListenerManager = new ExternalListenerManager(this.component);
    }

    @Override
    public void init(String serviceUnitName, String suRootPath) throws DeploymentException {
        super.init(serviceUnitName, suRootPath);
        try {
            this.externalListenerManager.createListeners(serviceUnitName,
                    this.getConsumesForSU(serviceUnitName));
        } catch (final PEtALSCDKException e) {
            final String msg = "Failed to initialize SU : " + serviceUnitName;
            this.logger.severe(msg);
            throw new DeploymentException(msg, e);
        }

    }

    @Override
    public void start(final String serviceUnitName) throws DeploymentException {
        super.start(serviceUnitName);
        try {
            this.externalListenerManager.startListening(serviceUnitName);
        } catch (final PEtALSCDKException e) {
            final String msg = "Failed to start SU : " + serviceUnitName;
            this.logger.severe(msg);
            throw new DeploymentException(msg, e);
        }
    }

    @Override
    public void stop(final String serviceUnitName) throws DeploymentException {
        // stop the consumption of services for the given SU, as said in the JBI
        // specifications
        try {
            this.externalListenerManager.stopListening(serviceUnitName);
        } catch (final PEtALSCDKException e) {
            final String msg = "Failed to stop SU : " + serviceUnitName;
            this.logger.severe(msg);
            throw new DeploymentException(msg, e);
        }
        super.stop(serviceUnitName);
    }

    @Override
    public String undeploy(final String serviceUnitName, final String serviceUnitRootPath)
            throws DeploymentException {
        this.externalListenerManager.removeListeners(serviceUnitName);

        return super.undeploy(serviceUnitName, serviceUnitRootPath);
    }

    /**
     * Get the Consumes sections for the given Service Unit.
     *  
     * @param serviceUnitName
     * @return
     * @throws DeploymentException
     */
    private List<Consumes> getConsumesForSU(String serviceUnitName) throws DeploymentException {
        final ServiceUnitDataHandler handler = this.getServiceUnitDataHandlers().get(serviceUnitName);
        if (handler != null) {
            return handler.getDescriptor().getServices().getConsumes();
        } else {
            throw new DeploymentException("No data handler for su : " + serviceUnitName
                    + ". SU not deployed.");
        }
    }

}
