/**
 * Copyright (c) 2009-2012 EBMWebsourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.component.framework.util;

import javax.xml.namespace.QName;

/**
 * @author Christophe Hamerling - EBM WebSourcing
 */
public class ServiceEndpointKey {

    private final QName serviceName;

    private final String endpointName;

    public ServiceEndpointKey(final QName serviceName, final String endpointName) {
        super();
        this.serviceName = serviceName;
        this.endpointName = endpointName;
    }

    @Override
    public String toString() {
        return "ServiceEndpointKey [endpointName=" + this.endpointName + ", serviceName="
        + this.serviceName + "]";
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        int hc;
        if (this.endpointName == null) {
            hc = 0;
        } else {
            hc = this.endpointName.hashCode();
        }
        result = prime * result + hc;
        if (this.serviceName == null) {
            hc = 0;
        } else {
            hc = this.serviceName.hashCode();
        }
        result = prime * result + hc;
        return result;
    }

    @Override
    public boolean equals(final Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (this.getClass() != obj.getClass()) {
            return false;
        }
        final ServiceEndpointKey other = (ServiceEndpointKey) obj;
        if (this.endpointName == null) {
            if (other.endpointName != null) {
                return false;
            }
        } else if (!this.endpointName.equals(other.endpointName)) {
            return false;
        }
        if (this.serviceName == null) {
            if (other.serviceName != null) {
                return false;
            }
        } else if (!this.serviceName.equals(other.serviceName)) {
            return false;
        }
        return true;
    }

}
