/**
 * Copyright (c) 2009-2012 EBMWebsourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.component.framework.util;

import java.util.Map;

import javax.xml.namespace.QName;

import org.ow2.petals.component.framework.api.Constants;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;
import org.w3c.dom.Element;

/**
 * WS-Addressing Helper class.
 * 
 * @author Christophe Hamerling - EBM WebSourcing
 * @deprecated use
 *             {@link com.ebmwebsourcing.wsstar.addressing.definition.WSAddressingFactory}
 *             with the key
 *             {@link org.ow2.petals.component.framework.api.WSStar.ADDRESSING_KEY}
 *             to manipulate the WS-Addressing property into a JBI exchange
 */
public class WSAHelper {

    /**
     * Constructor.
     */
    private WSAHelper() {
        // nothing
    }

    /**
     * Get the wsa:To data from the document fragment map
     * 
     * @param fragment
     * @return the URL as String or null
     */
    public static final String getTo(Map<String, DocumentFragment> map) {
        return getProperty(map, Constants.WSStar.Addressing.TO_QNAME.toString());
    }

    /**
     * Get the wsa:From data from the document fragment map
     * 
     * @param fragment
     * @return the URL as String or null
     */
    public static final String getFrom(Map<String, DocumentFragment> map) {
        return getProperty(map, Constants.WSStar.Addressing.FROM_QNAME.toString());
    }

    /**
     * Get the wsa:ReplyTo data from the document fragment map
     * 
     * @param fragment
     * @return the URL as String or null
     */
    public static final String getReplyTo(Map<String, DocumentFragment> map) {
        return getProperty(map, Constants.WSStar.Addressing.REPLY_TO_QNAME.toString());
    }

    /**
     * 
     * @param qname
     * @param text
     * @return
     */
    public static final DocumentFragment getElement(QName qname, String text) {
        DocumentFragment result = null;
        if (qname.equals(Constants.WSStar.Addressing.FROM_QNAME)) {
            result = getFromFragment(text);
        } else if (qname.equals(Constants.WSStar.Addressing.REPLY_TO_QNAME)) {
            result = getReplyToFragment(text);
        } else if (qname.equals(Constants.WSStar.Addressing.TO_QNAME)) {
            result = getToFragment(text);
        } 
        
        return result;
    }

    /**
     * 
     * @param text
     * @return
     */
    public static final DocumentFragment getToFragment(String text) {
        DocumentFragment result = createDocumentFragment(Constants.WSStar.Addressing.TO_QNAME);
        result.getFirstChild().setTextContent(text);
        result.normalize();
        return result;
    }

    /**
     * 
     * @param text
     * @return
     */
    public static final DocumentFragment getFromFragment(String text) {
        DocumentFragment result = createDocumentFragment(Constants.WSStar.Addressing.FROM_QNAME);
        Element address = createAddressElement(text);
        result.getFirstChild().appendChild(result.getOwnerDocument().importNode(address, true));
        result.normalize();
        return result;
    }

    /**
     * 
     * @param text
     * @return
     */
    public static final DocumentFragment getReplyToFragment(String text) {
        DocumentFragment result = createDocumentFragment(Constants.WSStar.Addressing.REPLY_TO_QNAME);
        Element address = createAddressElement(text);
        result.getFirstChild().appendChild(result.getOwnerDocument().importNode(address, true));
        result.normalize();
        return result;
    }

    /**
     * 
     * @param content
     * @return
     */
    protected static final Element createAddressElement(String content) {
        Document doc = JVMDocumentBuilders.newDocument();
        QName address = Constants.WSStar.Addressing.ADDRESS_QNAME;
        Element e = doc.createElementNS(address.getNamespaceURI(), address.getLocalPart());
        e.setPrefix(address.getPrefix());
        e.setTextContent(content);
        e.normalize();
        return e;
    }

    /**
     * 
     * @param documentName
     * @return
     */
    protected static final DocumentFragment createDocumentFragment(QName documentName) {
        DocumentFragment result = null;        
        Document doc = JVMDocumentBuilders.newDocument();
        Element elt = doc.createElementNS(documentName.getNamespaceURI(), documentName
                .getLocalPart());
        elt.setPrefix(documentName.getPrefix());
        result = doc.createDocumentFragment();
        result.appendChild(doc.importNode(elt, true));
        result.normalize();
        return result;
    }

    /**
     * 
     * @param map
     * @param name
     * @return
     */
    private static final String getProperty(Map<String, DocumentFragment> map, String name) {
        String result = null;
        DocumentFragment frag = map.get(name);
        if (frag != null && frag.getFirstChild() != null && frag.getFirstChild() instanceof Element) {
            result = frag.getTextContent();
        }
        return result;
    }
}
