/**
 * Copyright (c) 2011-2012 EBMWebsourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.component.framework.logger;

import static org.junit.Assert.assertEquals;
import static org.ow2.petals.commons.log.PetalsExecutionContext.FLOW_STEP_ENDPOINT_NAME_PROPERTY_NAME;
import static org.ow2.petals.commons.log.PetalsExecutionContext.FLOW_STEP_INTERFACE_NAME_PROPERTY_NAME;
import static org.ow2.petals.commons.log.PetalsExecutionContext.FLOW_STEP_OPERATION_NAME_PROPERTY_NAME;
import static org.ow2.petals.commons.log.PetalsExecutionContext.FLOW_STEP_SERVICE_NAME_PROPERTY_NAME;

import java.util.Map;

import org.junit.Test;
import org.ow2.petals.commons.log.TraceCode;

/**
 * @author Mathieu Carrolle - EBM WebSourcing
 */
public class ConsumeFlowStepBeginLogDataTest extends AbstractFlowLogDataTest {

    protected static final String TEST_FLOW_INTERFACE_NAME_STR = "{http://org.ow2.petals.test1}testFlowStepInterfaceName";

    protected static final String TEST_FLOW_SERVICE_NAME_STR = "{http://org.ow2.petals.test2}testFlowStepServiceName";

    protected static final String TEST_FLOW_OPERATION_NAME_STR = "{http://org.ow2.petals.test3}testFlowStepOperationName";

    protected static final String TEST_FLOW_ENDPOINT_NAME = "testFlowStepEndPointName";

    @Override
    @Test
    public void testConstructor() {
        TraceCode traceCode = TraceCode.CONSUME_FLOW_STEP_BEGIN;
        ConsumeFlowStepBeginLogData logData = new ConsumeFlowStepBeginLogData(
                TEST_FLOW_INSTANCE_ID, TEST_FLOW_STEP_ID, TEST_FLOW_INTERFACE_NAME_STR,
                TEST_FLOW_SERVICE_NAME_STR, TEST_FLOW_ENDPOINT_NAME, TEST_FLOW_OPERATION_NAME_STR);

        assertEquals(createExpectedLogData(traceCode), logData);
    }

    @Override
    protected Map<String, Object> createExpectedLogData(TraceCode traceCode) {
        Map<String, Object> expectedLogData = super.createExpectedLogData(traceCode);
        expectedLogData.put(FLOW_STEP_INTERFACE_NAME_PROPERTY_NAME,
                TEST_FLOW_INTERFACE_NAME_STR);
        expectedLogData.put(FLOW_STEP_SERVICE_NAME_PROPERTY_NAME,
                TEST_FLOW_SERVICE_NAME_STR);
        expectedLogData.put(FLOW_STEP_OPERATION_NAME_PROPERTY_NAME,
                TEST_FLOW_OPERATION_NAME_STR);
        expectedLogData.put(FLOW_STEP_ENDPOINT_NAME_PROPERTY_NAME,
                TEST_FLOW_ENDPOINT_NAME);
        return expectedLogData;
    }

}
