/**
 * Copyright (c) 2012 EBMWebsourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.component.framework.process;

import java.lang.ref.PhantomReference;
import java.lang.ref.Reference;
import java.lang.ref.ReferenceQueue;

import javax.jbi.component.ComponentContext;
import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;

import junit.framework.TestCase;

import org.easymock.IAnswer;
import org.easymock.classextension.EasyMock;
import org.ow2.petals.component.framework.AbstractComponent;
import org.ow2.petals.component.framework.jbidescriptor.generated.Component;
import org.ow2.petals.component.framework.jbidescriptor.generated.Identification;
import org.ow2.petals.component.framework.jbidescriptor.generated.Runtimepositivestrictint;
import org.ow2.petals.component.framework.jbidescriptor.generated.Runtimepositivestrictlong;
import org.ow2.petals.component.framework.jbidescriptor.generated.Settablepositivestrictint;
import org.ow2.petals.component.framework.jbidescriptor.generated.Settablepositivestrictlong;
import org.ow2.petals.component.framework.listener.AbstractJBIListener;
import org.ow2.petals.component.framework.monitoring.Monitoring;
import org.ow2.petals.component.framework.monitoring.key.ServiceProviderKey;
import org.ow2.petals.component.framework.monitoring.probe.MessageExchangeAcceptorPoolProbe;
import org.ow2.petals.probes.api.exceptions.MultipleProbesFactoriesFoundException;
import org.ow2.petals.probes.api.exceptions.NoProbesFactoryFoundException;
import org.ow2.petals.probes.api.key.ProbeKey;
import org.ow2.petals.probes.api.probes.CounterProbe;
import org.ow2.petals.probes.api.probes.DurationProbe;
import org.ow2.petals.probes.api.probes.KeyedCounterProbeWithExecutionStatus;
import org.ow2.petals.probes.api.probes.KeyedResponseTimeProbe;
import org.ow2.petals.probes.api.probes.macro.ObjectPoolProbe;
import org.ow2.petals.probes.api.probes.macro.ThreadPoolProbe;

public abstract class AbstractJBIManager<T> {

    protected final ReferenceQueue<T> queueGC = new ReferenceQueue<T>();

    protected void assertIsGarbaged(final PhantomReference<T> refMgr) {
        try {
            boolean found = false;
            Reference<?> phantRef;
            do {
                System.gc();
                phantRef = this.queueGC.remove(1000);
                if (phantRef == refMgr) {
                    found = true;
                }
            } while (!found && phantRef != null);
            if (!found) {
                TestCase.fail("JBI...Manager is not correctly garbaged");
            }
        } catch (final InterruptedException e) {
            TestCase.fail("JBI...Manager is not correctly garbaged !!!!");
        }
    }

    protected AbstractComponent createComponentMock(final Integer _acceptorsCoreSize,
            final Long _acceptorsStopMaxWait, final Integer _acceptorsRetryNb,
            final Long _acceptorsRetryWait, final Integer _messageProcessorMaxPoolSize,
            final Integer _processorCoreSize, final Integer _processorMaxSize,
            final Integer _processorKeepAliveTime, final Long _processorStopMaxWait,
            final Long _timeBetweenAsyncCleanerRuns) throws MessagingException,
            MultipleProbesFactoriesFoundException, NoProbesFactoryFoundException {

        final Runtimepositivestrictint acceptorPoolSize;
        if (_acceptorsCoreSize != null) {
            acceptorPoolSize = new Runtimepositivestrictint();
            acceptorPoolSize.setValue(_acceptorsCoreSize);
        } else {
            acceptorPoolSize = null;
        }

        final Runtimepositivestrictlong acceptorsStopMaxWait;
        if (_acceptorsStopMaxWait != null) {
            acceptorsStopMaxWait = new Runtimepositivestrictlong();
            acceptorsStopMaxWait.setValue(_acceptorsStopMaxWait);
        } else {
            acceptorsStopMaxWait = null;
        }

        final Settablepositivestrictint acceptorsRetryNb;
        if (_acceptorsRetryNb != null) {
            acceptorsRetryNb = new Settablepositivestrictint();
            acceptorsRetryNb.setValue(_acceptorsRetryNb);
        } else {
            acceptorsRetryNb = null;
        }

        final Settablepositivestrictlong acceptorsRetryWait;
        if (_acceptorsRetryWait != null) {
            acceptorsRetryWait = new Settablepositivestrictlong();
            acceptorsRetryWait.setValue(_acceptorsRetryWait);
        } else {
            acceptorsRetryWait = null;
        }

        final Runtimepositivestrictint messageProcessorMaxPoolSize;
        if (_messageProcessorMaxPoolSize != null) {
            messageProcessorMaxPoolSize = new Runtimepositivestrictint();
            messageProcessorMaxPoolSize.setValue(_messageProcessorMaxPoolSize);
        } else {
            messageProcessorMaxPoolSize = null;
        }

        final Runtimepositivestrictint processorPoolSize;
        if (_processorCoreSize != null) {
            processorPoolSize = new Runtimepositivestrictint();
            processorPoolSize.setValue(_processorCoreSize);
        } else {
            processorPoolSize = null;
        }

        final Runtimepositivestrictint processorMaxPoolSize;
        if (_processorMaxSize != null) {
            processorMaxPoolSize = new Runtimepositivestrictint();
            processorMaxPoolSize.setValue(_processorMaxSize);
        } else {
            processorMaxPoolSize = null;
        }

        final Runtimepositivestrictint processorKeepAliveTime;
        if (_processorKeepAliveTime != null) {
            processorKeepAliveTime = new Runtimepositivestrictint();
            processorKeepAliveTime.setValue(_processorKeepAliveTime);
        } else {
            processorKeepAliveTime = null;
        }

        final Runtimepositivestrictlong processorStopMaxWait;
        if (_processorStopMaxWait != null) {
            processorStopMaxWait = new Runtimepositivestrictlong();
            processorStopMaxWait.setValue(_processorStopMaxWait);
        } else {
            processorStopMaxWait = null;
        }

        final Settablepositivestrictlong timeBetweenAsyncCleanerRuns;
        if (_timeBetweenAsyncCleanerRuns != null) {
            timeBetweenAsyncCleanerRuns = new Settablepositivestrictlong();
            timeBetweenAsyncCleanerRuns.setValue(_timeBetweenAsyncCleanerRuns);
        } else {
            timeBetweenAsyncCleanerRuns = null;
        }

        final Identification identification = new Identification();
        identification.setName("my-component");

        final AbstractJBIListener jbiListener = EasyMock.createNiceMock(AbstractJBIListener.class);
        jbiListener.init();
        EasyMock.expectLastCall();
        EasyMock.replay(jbiListener);

        final Component componentCfg = EasyMock.createMock(Component.class);
        EasyMock.expect(componentCfg.getIdentification()).andReturn(identification).anyTimes();
        EasyMock.expect(componentCfg.getAcceptorPoolSize()).andReturn(acceptorPoolSize).anyTimes();
        EasyMock.expect(componentCfg.getAcceptorRetryNumber()).andReturn(acceptorsRetryNb)
                .anyTimes();
        EasyMock.expect(componentCfg.getAcceptorRetryWait()).andReturn(acceptorsRetryWait)
                .anyTimes();
        EasyMock.expect(componentCfg.getAcceptorStopMaxWait()).andReturn(acceptorsStopMaxWait)
                .anyTimes();
        EasyMock.expect(componentCfg.getMessageProcessorMaxPoolSize())
                .andReturn(messageProcessorMaxPoolSize).anyTimes();
        EasyMock.expect(componentCfg.getProcessorPoolSize()).andReturn(processorPoolSize)
                .anyTimes();
        EasyMock.expect(componentCfg.getProcessorMaxPoolSize()).andReturn(processorMaxPoolSize)
                .anyTimes();
        EasyMock.expect(componentCfg.getProcessorKeepAliveTime()).andReturn(processorKeepAliveTime)
                .anyTimes();
        EasyMock.expect(componentCfg.getProcessorStopMaxWait()).andReturn(processorStopMaxWait)
                .anyTimes();
        EasyMock.expect(componentCfg.getTimeBeetweenAsyncCleanerRuns()).andReturn(timeBetweenAsyncCleanerRuns)
                .anyTimes();
        EasyMock.expect(componentCfg.getJbiListenerClassName())
                .andReturn(jbiListener.getClass().getName()).anyTimes();
        EasyMock.replay(componentCfg);

        final DeliveryChannel deliveryChannel = EasyMock.createMock(DeliveryChannel.class);
        // No message exchange factory is needed by the test
        EasyMock.expect(deliveryChannel.createExchangeFactory()).andReturn(null).anyTimes();
        EasyMock.expect(deliveryChannel.accept()).andAnswer(new IAnswer<MessageExchange>() {

            @Override
            public MessageExchange answer() throws Throwable {
                Thread.sleep(5000);
                return null;
            }
        }).anyTimes();
        EasyMock.replay(deliveryChannel);

        final ComponentContext context = EasyMock.createMock(ComponentContext.class);
        EasyMock.expect(context.getComponentName()).andReturn("my-component").anyTimes();
        // No transaction manager is needed by the test
        EasyMock.expect(context.getTransactionManager()).andReturn(null).anyTimes();
        EasyMock.replay(context);

        final Monitoring monitoringMock = EasyMock.createNiceMock(Monitoring.class);
        this.addProbeMocks(monitoringMock);
        EasyMock.replay(monitoringMock);

        final AbstractComponent componentMock = EasyMock.createMock(AbstractComponent.class);
        EasyMock.expect(componentMock.getComponentConfiguration()).andReturn(componentCfg)
                .anyTimes();
        EasyMock.expect(componentMock.getChannel()).andReturn(deliveryChannel).anyTimes();
        EasyMock.expect(componentMock.getMonitoringBean()).andReturn(monitoringMock).anyTimes();
        // No service unit manager is needed by the test
        EasyMock.expect(componentMock.getServiceUnitManager()).andReturn(null).anyTimes();
        // No async manager is needed by the test
        EasyMock.expect(componentMock.getAsyncManager()).andReturn(null).anyTimes();
        EasyMock.expect(componentMock.getContext()).andReturn(context).anyTimes();
        EasyMock.replay(componentMock);

        return componentMock;
    }

    private void addProbeMocks(final Monitoring monitoringMock) {
        final MessageExchangeAcceptorPoolProbe messageExchangeAcceptorPoolProbeMock = EasyMock
                .createMock(MessageExchangeAcceptorPoolProbe.class);
        EasyMock.expect(monitoringMock.getMessageExchangeAcceptorPoolProbe())
                .andReturn(messageExchangeAcceptorPoolProbeMock).anyTimes();

        final ThreadPoolProbe threadPoolProbeMock = EasyMock.createMock(ThreadPoolProbe.class);
        EasyMock.expect(monitoringMock.getMessageExchangeProcessorThreadPoolProbe())
                .andReturn(threadPoolProbeMock).anyTimes();

        final ObjectPoolProbe mexProcessorObjectPoolProbeMock = EasyMock
                .createMock(ObjectPoolProbe.class);
        EasyMock.expect(monitoringMock.getMessageExchangeProcessorObjectPoolProbe())
                .andReturn(mexProcessorObjectPoolProbeMock).anyTimes();

        final CounterProbe mexProcessorObjectPoolExhaustionProbeMock = EasyMock
                .createMock(CounterProbe.class);
        EasyMock.expect(monitoringMock.getMessageExchangeProcessorObjectPoolExhaustionProbe())
                .andReturn(mexProcessorObjectPoolExhaustionProbeMock).anyTimes();

        final DurationProbe mexProcessorDurationProbeMock = EasyMock
                .createMock(DurationProbe.class);
        EasyMock.expect(monitoringMock.getMessageExchangeProcessorDurationProbe())
                .andReturn(mexProcessorDurationProbeMock).anyTimes();

        final KeyedResponseTimeProbe<ServiceProviderKey> serviceProviderResponseTimesProbeMock = EasyMock
                .createMock(KeyedResponseTimeProbe.class);
        EasyMock.expect(monitoringMock.getServiceProviderResponseTimesProbe())
                .andReturn(serviceProviderResponseTimesProbeMock).anyTimes();

        final KeyedCounterProbeWithExecutionStatus<ProbeKey> serviceProviderInvokationsProbeMock = EasyMock
                .createMock(KeyedCounterProbeWithExecutionStatus.class);
        EasyMock.expect(monitoringMock.getServiceProviderInvokationsProbe())
                .andReturn(serviceProviderInvokationsProbeMock).anyTimes();

    }


}
