/**
 * Copyright (c) 2008-2012 EBMWebsourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.component.framework.process.async;

import java.lang.reflect.InvocationTargetException;
import java.util.logging.Logger;

import javax.jbi.JBIException;

import junit.framework.TestCase;

import org.junit.Test;
import org.ow2.petals.component.framework.AbstractComponent;
import org.ow2.petals.component.framework.mbean.RuntimeConfigurationNotifier;
import org.ow2.petals.component.framework.process.AbstractJBIManager;
import org.ow2.petals.component.framework.process.JBIProcessorManager;
import org.ow2.petals.probes.api.exceptions.MultipleProbesFactoriesFoundException;
import org.ow2.petals.probes.api.exceptions.NoProbesFactoryFoundException;

import com.ebmwebsourcing.easycommons.lang.reflect.ReflectionHelper;

public class AsyncMessageManagerTest extends AbstractJBIManager<AsyncMessageManagerTest> {
    
    private final static Logger LOG = Logger.getLogger(AsyncMessageManagerTest.class.getName());

    /**
     * Checks the configuration against a value of time between async cleaner
     * runs not set. Expected result: the default value is used.
     * 
     */
    @Test
    public void testGetTimeBeetweenAsyncCleanerRuns_000() throws JBIException,
            IllegalArgumentException, IllegalAccessException, InvocationTargetException,
            MultipleProbesFactoriesFoundException, NoProbesFactoryFoundException {

        final AbstractComponent componentMock = this.createComponentMock(10, null, -10, null, null,
                10, null, null, null, null);

        final JBIProcessorManager procMgr = new JBIProcessorManager(componentMock,
                new RuntimeConfigurationNotifier(null, null, null), LOG);

        final AsyncMessageManager asyncMsgMgr = new AsyncMessageManager(componentMock, procMgr, LOG);
        
        TestCase.assertEquals(
                "Time between async cleaner runs is not set to the default value.",
                AsyncMessageManager.DEFAULT_TIME_BETWEEN_ASYNC_CLEANER_RUNS,
                ((Long) ReflectionHelper.invokePrivateMethod(asyncMsgMgr,
                        "getTimeBeetweenAsyncCleanerRuns", new Object[] {}, new Class<?>[] {}))
                        .longValue());
    }
    
    /**
     * Checks the configuration against a value of time between async cleaner
     * runs set to a negative value. Expected result: the default value is used.
     * 
     */
    @Test
    public void testGetTimeBeetweenAsyncCleanerRuns_001() throws JBIException,
            IllegalArgumentException, IllegalAccessException, InvocationTargetException,
            MultipleProbesFactoriesFoundException, NoProbesFactoryFoundException {

        final AbstractComponent componentMock = this.createComponentMock(10, null, -10, null, null,
                10, null, null, null, -345l);

        final JBIProcessorManager procMgr = new JBIProcessorManager(componentMock,
                new RuntimeConfigurationNotifier(null, null, null), LOG);

        final AsyncMessageManager asyncMsgMgr = new AsyncMessageManager(componentMock, procMgr, LOG);

        TestCase.assertEquals("Time between async cleaner runs is not set to the default value.",
                AsyncMessageManager.DEFAULT_TIME_BETWEEN_ASYNC_CLEANER_RUNS,
                ((Long) ReflectionHelper.invokePrivateMethod(asyncMsgMgr,
                        "getTimeBeetweenAsyncCleanerRuns", new Object[] {}, new Class<?>[] {}))
                        .longValue());
    }

    /**
     * Checks the configuration against a value of time between async cleaner
     * runs set to 0. Expected result: the default value is used.
     * 
     */
    @Test
    public void testGetTimeBeetweenAsyncCleanerRuns_002() throws JBIException,
            IllegalArgumentException, IllegalAccessException, InvocationTargetException,
            MultipleProbesFactoriesFoundException, NoProbesFactoryFoundException {

        final AbstractComponent componentMock = this.createComponentMock(10, null, -10, null, null,
                10, null, null, null, 0l);

        final JBIProcessorManager procMgr = new JBIProcessorManager(componentMock,
                new RuntimeConfigurationNotifier(null, null, null), LOG);

        final AsyncMessageManager asyncMsgMgr = new AsyncMessageManager(componentMock, procMgr, LOG);

        TestCase.assertEquals("Time between async cleaner runs is not set to the default value.",
                AsyncMessageManager.DEFAULT_TIME_BETWEEN_ASYNC_CLEANER_RUNS,
                ((Long) ReflectionHelper.invokePrivateMethod(asyncMsgMgr,
                        "getTimeBeetweenAsyncCleanerRuns", new Object[] {}, new Class<?>[] {}))
                        .longValue());
    }

    /**
     * Checks the configuration against a value of time between async cleaner
     * runs set to a valid value. Expected result: the configured value is used.
     * 
     */
    @Test
    public void testGetTimeBeetweenAsyncCleanerRuns_003() throws JBIException,
            IllegalArgumentException, IllegalAccessException, InvocationTargetException,
            MultipleProbesFactoriesFoundException, NoProbesFactoryFoundException {

        final long expectedTimeBeetweenAsyncCleanerRuns = 5000l;
        final AbstractComponent componentMock = this.createComponentMock(10, null, -10, null, null,
                10, null, null, null, expectedTimeBeetweenAsyncCleanerRuns);

        final JBIProcessorManager procMgr = new JBIProcessorManager(componentMock,
                new RuntimeConfigurationNotifier(null, null, null), LOG);

        final AsyncMessageManager asyncMsgMgr = new AsyncMessageManager(componentMock, procMgr, LOG);

        TestCase.assertEquals("Time between async cleaner runs is not set to the expected value.",
                expectedTimeBeetweenAsyncCleanerRuns, ((Long) ReflectionHelper.invokePrivateMethod(
                        asyncMsgMgr, "getTimeBeetweenAsyncCleanerRuns", new Object[] {},
                        new Class<?>[] {})).longValue());

    }
}
