/**
 * Copyright (c) 2009-2012 EBMWebsourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.component.framework.su;

import static org.junit.Assert.fail;

import java.lang.reflect.InvocationTargetException;
import java.net.URL;

import javax.xml.namespace.QName;

import org.junit.Before;
import org.junit.Test;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.api.Description;
import org.ow2.easywsdl.wsdl.api.WSDLException;
import org.ow2.petals.component.framework.jbidescriptor.generated.Provides;
import org.ow2.petals.component.framework.util.WSDLUtilImpl;

import com.ebmwebsourcing.easycommons.lang.reflect.ReflectionHelper;

/**
 * @author Christophe Hamerling - EBM WebSourcing
 */
public class AbstractServiceUnitManagerTest {

    Description description = null;

    /**
     * @param name
     */
    public AbstractServiceUnitManagerTest() {
        // nothing
    }

    /**
     * {@inheritDoc}
     */
    @Before
    public void setUp() throws Exception {
        final URL url = AbstractServiceUnitManagerTest.class.getResource("/ClockService.wsdl");
        if (url == null) {
            fail("WSDL not found!");
        }
        try {
            this.description = WSDLUtilImpl.createWsdlDescription(url, false);
        } catch (final WSDLException e) {
            fail(e.getMessage());
        }
    }

    @Test
    public void testAllMatch() throws IllegalArgumentException, IllegalAccessException,
            InstantiationException {
        final Provides provides = new Provides();
        provides.setEndpointName("ClockEndpoint");
        provides.setServiceName(QName.valueOf("{http://petals.ow2.org}ClockService"));
        provides.setInterfaceName(QName.valueOf("{http://petals.ow2.org}Clock"));

        try {
            ReflectionHelper.invokePrivateMethod(AbstractServiceUnitManager.class, null,
                    "validateDescription", new Object[] { this.description, provides },
                    new Class[] { Description.class, Provides.class });
        } catch (final InvocationTargetException e) {
            fail(e.getMessage());
        }
    }

    @Test
    public void testBadInterface() throws IllegalArgumentException, IllegalAccessException {
        final Provides provides = new Provides();
        provides.setEndpointName("ClockEndpoint");
        provides.setServiceName(QName.valueOf("{http://petals.ow2.org}ClockService"));
        provides.setInterfaceName(QName.valueOf("{http://petals.ow2.org}Clock2"));

        try {
            ReflectionHelper.invokePrivateMethod(AbstractServiceUnitManager.class, null,
                    "validateDescription", new Object[] { this.description, provides },
                    new Class[] { Description.class, Provides.class });
            fail();
        } catch (final InvocationTargetException e) {
            // / ok
        }
    }

    @Test
    public void testBadEndpoint() throws IllegalArgumentException, IllegalAccessException {
        final Provides provides = new Provides();
        provides.setEndpointName("ClockEndpoint");
        provides.setServiceName(QName.valueOf("{http://petals.ow2.org}ClockService2"));
        provides.setInterfaceName(QName.valueOf("{http://petals.ow2.org}Clock"));

        try {
            ReflectionHelper.invokePrivateMethod(AbstractServiceUnitManager.class, null,
                    "validateDescription", new Object[] { this.description, provides },
                    new Class[] { Description.class, Provides.class });
            fail();
        } catch (final InvocationTargetException e) {
            // test ok
        }
    }

    @Test
    public void testBadService() throws IllegalArgumentException, IllegalAccessException {
        final Provides provides = new Provides();
        provides.setEndpointName("ClockEndpoint2");
        provides.setServiceName(QName.valueOf("{http://petals.ow2.org}ClockService"));
        provides.setInterfaceName(QName.valueOf("{http://petals.ow2.org}Clock"));

        try {
            ReflectionHelper.invokePrivateMethod(AbstractServiceUnitManager.class, null,
                    "validateDescription", new Object[] { this.description, provides },
                    new Class[] { Description.class, Provides.class });
            fail();
        } catch (final InvocationTargetException e) {
            // test ok
        }
    }

}
