/**
 * Copyright (c) 2006-2012 EBMWebsourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.component.framework.util;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import java.util.LinkedList;
import java.util.List;

import org.junit.Test;
import org.ow2.petals.component.framework.util.FileNamePatternUtil.RegexFileNameFilter;

/**
 * @author Adrien Louis - EBM WebSourcing
 */
public class FileNamePatternUtilTest {

    @Test
    public void testRegexToWildchar() {
        assertEquals("?.XXX?", FileNamePatternUtil.getInstance().regexToWildchar(".\\.XXX."));
    }

    @Test
    public void testWildcharToRegex() {
        assertEquals(".*\\..*", FileNamePatternUtil.getInstance().wildcharToRegex("*.*"));
        assertEquals(".\\.XXX.", FileNamePatternUtil.getInstance().wildcharToRegex("?.XXX?"));
    }

    @Test
    public void testBuildFileNameFilterFromRegex() {
        RegexFileNameFilter a = FileNamePatternUtil.getInstance().buildFileNameFilterFromRegex(
                ".*\\.a");
        assertTrue(a.accept(null, "XXX.a"));
        assertTrue(a.accept(null, ".a"));
        assertFalse(a.accept(null, "a"));
        assertFalse(a.accept(null, "XXX.aX"));
    }

    @Test
    public void testBuildFileNameFilterFromWildChar() {
        RegexFileNameFilter a = FileNamePatternUtil.getInstance().buildFileNameFilterFromWildChar(
                "*.a");
        assertTrue(a.accept(null, "XXX.a"));
        assertTrue(a.accept(null, ".a"));
        assertFalse(a.accept(null, "a"));
        assertFalse(a.accept(null, "XXX.aX"));
        a = FileNamePatternUtil.getInstance().buildFileNameFilterFromWildChar("X?X.X?X");
        assertTrue(a.accept(null, "XXX.XXX"));
        assertTrue(a.accept(null, "XzX.XzX"));
        assertFalse(a.accept(null, "ZZZ.ZZZ"));
        a = FileNamePatternUtil.getInstance().buildFileNameFilterFromWildChar("*.*");
        assertTrue(a.accept(null, "X.X"));
        assertFalse(a.accept(null, "XXX"));
    }

    @Test
    public void testBuildFileNameFilterFromFilters() {
        RegexFileNameFilter a = FileNamePatternUtil.getInstance().buildFileNameFilterFromWildChar(
                "*.a");
        RegexFileNameFilter b = FileNamePatternUtil.getInstance().buildFileNameFilterFromWildChar(
                "*.b");
        List<RegexFileNameFilter> ab = new LinkedList<RegexFileNameFilter>();
        ab.add(a);
        ab.add(b);
        RegexFileNameFilter abf = FileNamePatternUtil.getInstance().buildFileNameFilterFromFilters(
                ab);
        assertTrue(abf.accept(null, "adri.a"));
        assertTrue(abf.accept(null, "adri.b"));
        assertFalse(abf.accept(null, "adri.c"));
    }
}
